/*
 * File: vr.c
 *
 *	Here's a client that supports virtual room functionality
 *	There are up to six rooms that can be used with up to MAX_CLIENTS [256] clients at once.
 *	The rooms are loaded by toggling to that room and starting the clients for that room.
 *	
 *	Basically, This client performs sesion-managament type functionality via ICCCM properties
 *	and keeps track of what clients are in which rooms and then sets the states between
 *	normal, iconic and withdrawn. 
 *
 *
 * vr1.1  7/1/92 :
 * Thanks to Jay Schmidgall jay@vnet.ibm.com for generalizing rooms ad anfinitum 
 *	and resize handler!
 * Thanks to Scott Grosch garath@engin.umich.edu for exit handler (needed badly)
 *
 * vr1.0 Created: 5/1/92 Richard Mauri rmauri@netcom.com
 *	Trimble Navigation Ltd.
 *
 Copyright (c) 1992 by Richard Mauri Trimble Navigation Ltd.

 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
 supporting documentation.

 This file is provided AS IS with no warranties of any kind.  The author
 shall have no liability with respect to the infringement of copyrights,
 trade secrets or any patents by this file or any part thereof.  In no
 event will the author be liable for any lost revenue or profits or
 other special, indirect and consequential damages.
 */

#include "vr.h"
#include <signal.h>

static int current_room = 0;
static Widget previous_room;
static String *all_rooms;
static int n_all_rooms;

static int nvri, ntvri;
static VRInfo vri[MAX_CLIENTS], tvri[MAX_CLIENTS];

static Display *dpy;
static Window wdw;
static Cursor watch_cur;
static Widget pb;			/* Push Button used to popup dialog to re-label room buttons */
static Widget *roompb;			/* Toggle buttons to switch between rooms */
static Widget dialog_form;		/* This is the configuration dialog (a form not a shell!) */
static Widget *roomtext;		/* Text fields used to re-label room buttons */
static String *room_label;		/* Current labels used for room buttons */
static XtAppContext context;
static AppData app_data;
static RoomDataPtr room_data;
static char title[80];
static Bool mwm_running;
static int ALLROOMS;
static int MAX_ROOMS;
static int loop;
	
/*
 * Forward declarations for internal procedures
 */
static void InitRoomAux();
static void InitRoom();
static void SetString();
static void SetWindowVisibility();
static void switch_rooms();
static void switch_rooms_aux();
static void SetAllVroomProperty();
static void exit_receiver();
static void InitRoomAux();
static Bool WindowNotInList();
static void up();
static void SaveRoomTitles();
static void LoadLabel();
static void InitRoomTitles();
static Bool SetRoomLabel();
static Bool GetWMState();
static Bool GetWMName();
static void SetString();
static void Usage();
static void TopLevelInterface();
static void menucb();
static void LoadPrefs();
static void InitInterface();
static void GetAllRoomPrefs();
static void DialogInterface();

/* External procedures */
extern String ExpandFileName();

void exit_vr() {
     int i;

     for (i = 0;i < nvri; i++)
          XMapWindow(dpy, vri[i].window);

     exit_receiver(1);
}

void main(argc, argv)
int argc;
char *argv[];
{
 	int loop;
 	char cmdN[30];
 	char cmdC[30];
 	char pswdN[30];
 	char pswdC[30];
 	char colorN[30];
 	char colorC[30];
 	RoomData get_room_data;
   	Widget TopLevel = XtVaAppInitialize(&context, "Vr", options,
  		XtNumber(options), &argc, argv, NULL, NULL);
  
  	XtGetApplicationResources(TopLevel, &app_data, resources, XtNumber(resources), NULL, 0);
 	/* Don't let bogus values through for nrooms (must be 1 - 6) */
 	
 	if (app_data.nrooms < 0) app_data.nrooms = 1;
  
 	ALLROOMS = app_data.nrooms;
 	MAX_ROOMS = app_data.nrooms;
   	if (argc > 1) Usage();
  
 	/* get some memory */
 	room_data = (RoomDataPtr) XtMalloc(sizeof(RoomData) * MAX_ROOMS);
 	roompb = (Widget*) XtMalloc(sizeof(Widget) * MAX_ROOMS);
 	roomtext = (Widget*) XtMalloc(sizeof(Widget) * MAX_ROOMS);
 	room_label = (char**) XtMalloc(sizeof(char*) * MAX_ROOMS);
 	for (loop = 0; loop < MAX_ROOMS; loop++)
 	    room_label[loop] = (char*) XtMalloc(sizeof(char)*STRING_LENGTH);
  	/* read room resources */
 	for (loop = 0; loop < MAX_ROOMS; loop++)
 	{
 	    sprintf(colorN,	XtNroomColor,	loop+1);
 	    sprintf(colorC,	XtCroomColor,	loop+1);
 	    sprintf(cmdN,	XtNroom_cmds,	loop+1);
 	    sprintf(cmdC,	XtCroom_cmds,	loop+1);
 	    sprintf(pswdN,	XtNroom_passwd, loop+1);
 	    sprintf(pswdC,	XtCroom_passwd, loop+1);
  	    room_resources[0].resource_name	= colorN;
 	    room_resources[0].resource_class	= colorC;
 	    room_resources[0].resource_type	= XtRPixel;
 	    room_resources[0].resource_size	= sizeof(Pixel);
 	    room_resources[0].resource_offset	= XtOffset(RoomDataPtr, room_color);
 	    room_resources[0].default_type	= XtRString;
 	    room_resources[0].default_addr	= "grey40";
  	
 	    room_resources[1].resource_name	= cmdN;
 	    room_resources[1].resource_class	= cmdC;
 	    room_resources[1].resource_type	= XtRString;
 	    room_resources[1].resource_size	= sizeof(String);
 	    room_resources[1].resource_offset	= XtOffset(RoomDataPtr, room_cmds);
 	    room_resources[1].default_type	= XtRString;
 	    room_resources[1].default_addr	= NULL;
  	
 	    room_resources[2].resource_name	= pswdN;
 	    room_resources[2].resource_class	= pswdC;
 	    room_resources[2].resource_type	= XtRString;
 	    room_resources[2].resource_size	= sizeof(String);
 	    room_resources[2].resource_offset	= XtOffset(RoomDataPtr, room_passwd);
 	    room_resources[2].default_type	= XtRString;
 	    room_resources[2].default_addr	= NULL;
 	    	
 	    XtVaGetApplicationResources(TopLevel, &get_room_data,
 					room_resources,
 					XtNumber(room_resources),
 					NULL);
 	
 	    room_data[loop].room_color = get_room_data.room_color;
 	    room_data[loop].room_cmds = XtNewString(get_room_data.room_cmds);
 	    room_data[loop].room_passwd = XtNewString(get_room_data.room_passwd);
 	}
 	
  	mwm_running = XmIsMotifWMRunning(TopLevel);
  
  	dpy = XtDisplayOfObject(TopLevel);

	TopLevelInterface(TopLevel);

	InitInterface();

	XtRealizeWidget(TopLevel);

	wdw = XtWindowOfObject(TopLevel);
	watch_cur = XCreateFontCursor(dpy, XC_watch);
	
	if (GetWMName(XtWindowOfObject(TopLevel), title)) SetString(pb, title);

	GetAllRoomPrefs();

	InitRoom(True, 0);

	SetAllVroomProperty();

	xs_keyreceiver(VRNAME, TopLevel, up, 0);

	/*
	 * Currently, we alsways start out at room 1
	 */
	if (room_data[0].room_cmds) system(room_data[0].room_cmds);
	previous_room = roompb[0];

	XtAppAddTimeOut(context, 1000, LoadPrefs, 0);

	/* If vr exits then don't leave windows withdrawn forever.... */

	signal(SIGHUP,exit_vr);
	signal(SIGINT,exit_vr);
	signal(SIGQUIT,exit_vr);
	signal(SIGTERM,exit_vr);
	signal(SIGSEGV,exit_vr);
	signal(SIGBUS,exit_vr);

	XtAppMainLoop(context);
}

static void ehTopLevelResize(w, client_data, evt)
Widget w;
XtPointer client_data;
XEvent *evt;
{
    int		numbuttons, leftover, numcolumns, total;
    int		min_spacing = 2, min_margin = 2;
    Dimension	new_height, spacing, total_button_height;
    Dimension	total_for_buttons;
    Dimension	button_height, shadow, hilite, border;
    WidgetList	children;
    if (ConfigureNotify == evt->type)
    {
	new_height = evt->xconfigure.height;
	XtVaGetValues(w,
		      XmNnumChildren, &numbuttons,
		      XmNchildren, &children,
		      NULL);
    }	
    else if (MapNotify == evt->type)
	XtVaGetValues(w,
		      XmNheight, &new_height,
		      XmNnumChildren, &numbuttons,
		      XmNchildren, &children,
		      NULL);
    else return;
    XtVaGetValues(children[0],
		  XmNheight, &button_height,
		  XmNshadowThickness, &shadow,
		  XmNhighlightThickness, &hilite,
		  XmNborderWidth, &border,
		  NULL);
    total_button_height = button_height + 2*border + 2*shadow;
    total_for_buttons = numbuttons*total_button_height + min_spacing*numbuttons;
    total = total_for_buttons + 2*min_margin;
    if (new_height < total)
    {
	if (0 >= (numcolumns = new_height/button_height)) numcolumns = 1;
	else if (numcolumns > numbuttons) numcolumns = numbuttons;
    }
    else
    {
	numcolumns = numbuttons;
	leftover = (new_height - (int)button_height*numcolumns)/(numcolumns+1);
    }
    XtVaSetValues(w, XmNnumColumns, numcolumns, NULL);
}
static void TopLevelInterface(TopLevel)
Widget TopLevel;
{
    Widget form, rc;
    int n;
    Arg args[12];
    char name[10];

    n = 0;
    form = XmCreateForm(TopLevel, "form", args, n);
    XtManageChild(form);

    n = 0;
    XtSetArg(args[n], XmNrightAttachment,  XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftAttachment,   XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;

    pb = XmCreatePushButton(form, "pb", args, n);
    if (app_data.want_dialog) {
	XtManageChild(pb);
	XtAddCallback(pb, XmNactivateCallback, menucb, 0);
    }

    n = 0;
    XtSetArg(args[n], XmNadjustLast, False); n++;
    XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
    XtSetArg(args[n], XmNradioBehavior, True); n++;
    XtSetArg(args[n], XmNradioAlwaysOne, True); n++;
    XtSetArg(args[n], XmNrightAttachment,  XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftAttachment,   XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNtopWidget, pb); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNentryAlignment, XmALIGNMENT_CENTER); n++;
    rc = XmCreateRowColumn(form, "rc", args, n);
    XtManageChild(rc);

    /* handles repositioning of buttons on resize */
    XtAddEventHandler(rc, StructureNotifyMask, False, ehTopLevelResize, NULL);
    /* Be careful here when adding new resource args to consider the magic "3"
     * for the number of args for the other toggle buttons.
     */
    XtSetArg(args[0], XmNmarginWidth, 50);
    XtSetArg(args[1], XmNindicatorOn, False);
    XtSetArg(args[2], XmNshadowThickness, 2);
    XtSetArg(args[3], XmNrecomputeSize,	False);
    XtSetArg(args[4], XmNset, True);
    roompb[0] = XmCreateToggleButton(rc, "pb1", args, 5);
    XtManageChild(roompb[0]);
    XtAddCallback(roompb[0], XmNvalueChangedCallback, switch_rooms, 0);

    for (loop = 1; loop < MAX_ROOMS; loop++)
    {
	sprintf(name, "pb%d", loop+1);
	roompb[loop] = XmCreateToggleButton(rc, name, args, 4);
	XtManageChild(roompb[loop]);
	XtAddCallback(roompb[loop], XmNvalueChangedCallback, switch_rooms, loop);
    }

}

static void InitInterface()
{
        InitRoomTitles();

	for (loop = 0; loop < MAX_ROOMS; loop++)
	    XtVaSetValues(roompb[loop], XmNbackground, room_data[loop].room_color, NULL);
}

static void SetWindowVisibility(vri)
VRInfo *vri;
{
    XWMHints wm_hints;

    if (vri->room == ALLROOMS) return; /* Always Mapped */

    if (vri->room == current_room) {

	    wm_hints.initial_state = vri->wm_state.state;
	    wm_hints.flags = StateHint;
	    XSetWMHints(dpy, vri->window, &wm_hints);

	switch (vri->wm_state.state) {
	  case WithdrawnState:
		XWithdrawWindow(dpy, vri->window, DefaultScreen(dpy));
		break;
	  case NormalState:
		if (!mwm_running) {
		    XSizeHints hints;

		    hints.x = vri->x;
		    hints.y = vri->y;
		    hints.width = vri->width;
		    hints.height = vri->height;
		    hints.flags = PPosition|PSize;
		    XMoveResizeWindow(dpy, vri->window, vri->x, vri->y, 
			(unsigned int) vri->width, (unsigned int) vri->height);
		    XmuUpdateMapHints(dpy, vri->window, &hints);
		}
		XMapWindow(dpy, vri->window);
		break;
	  case IconicState:
		XMapWindow(dpy, vri->window); 
		/* 
		 * This map and iconify combination causes "flashing" on 
		 * some systems when windows go from withdrawn to iconic state.
		 * I don't know how to avoid this so I live with it.
		 */
		XIconifyWindow(dpy, vri->window, DefaultScreen(dpy));
		break;
	}
    }
    else XWithdrawWindow(dpy, vri->window, DefaultScreen(dpy));

    XFlush(dpy); /* Now dammit! */
}

static void switch_rooms_aux(room)
int room;
{
	int i;
	char cmd[80];

	XDefineCursor(dpy, wdw, watch_cur);
	
	XSync(dpy, True);

	if (room_data[room].room_passwd && room_data[room].room_passwd[0]) {
	    sprintf(cmd, "xpasswd -passwd %s +system_modal +one_try",
		    room_data[room].room_passwd);
	    if (system(cmd)) {
		XmToggleButtonSetState(previous_room, True, True);
		XUndefineCursor(dpy, wdw);
		return;
	    }
	    sleep (1); /* give xpasswd a chance to exit (we are not synchronous) */
	}

	InitRoom(False, current_room);

	current_room = (int) room;
	
	if (room_data[current_room].room_cmds) system(room_data[current_room].room_cmds);

	for (i = 0; i < nvri; i++) {

	    /*
	     * Before we change visibility states for all the current toplevel windows:
	     * Set each toplevel window (except vr itself) vroom property so each app 
	     * can determine what room that window is in and what is the current room.
	     */
	    if (strcasecmp(vri[i].name, title))  {
		SetVroomProperty(dpy, vri[i].window, vri[i].room, current_room);
		SetWindowVisibility(&vri[i]);
	    }
	}
	
	XUndefineCursor(dpy, wdw);
}

static void SetAllVroomProperty()
{
    int i;

    for (i = 0; i < nvri; i++) {
	SetVroomProperty(dpy, vri[i].window, vri[i].room, current_room); /* vrutil (omilib) routine */
    }
}

static void switch_rooms(w, tag, cbs)
Widget w;
XtPointer tag;
XmToggleButtonCallbackStruct *cbs;
{
	if (cbs->set == False) {
		previous_room = w;
		return; /* only worry about it when they are set */
	}

	if (current_room != (int) tag) switch_rooms_aux((int) tag);
}

static void exit_receiver(code)
int code;
{
    xs_keyreceiver_done(dpy, VRNAME);
    exit (code);
}

static void InitRoomAux(window, init, room)
Window window;
Bool init;
int room;
{
    int i, j;
    Window          topLevelWindow, rootWindow, *childList, parentWindow;
    unsigned int    childListLen;
    WM_STATE	    wmstate;
    char name[80];

    if (init) {
	ntvri = 0;
	memset((String) tvri, 0, sizeof(tvri)); /* reinitialize */
	for (i = 0; i < MAX_CLIENTS; i++) tvri[i].room = ALLROOMS;
    }
    init = False;

	/* 
	 * All the clients currently running by default are mapped or iconic
	 * only in room. Lets set that up now...
	 */

	/* get a list of children of the root window */

    if (XQueryTree(dpy, window, &rootWindow,
	&parentWindow, &childList, &childListLen) != 1) {
		fprintf(stderr, "Can't query window tree\n");
		exit_receiver(1);
    }

    if (childListLen == 0) {
	if (childList) XFree(childList);
	return;
    }

    for (i = 0; i < childListLen; i++) {
	memset((String) &wmstate, 0, sizeof(WM_STATE));
	if (! GetWMState(childList[i], &wmstate)) {
		InitRoomAux(childList[i], init, room);
		continue;
	}

	if (childList[i] == DefaultRootWindow(dpy));

	else if (GetWMName(childList[i], name)) {
	    Bool its_in_all_rooms = False;

	    for (j = 0; j < n_all_rooms; j++) {
		if ( (! its_in_all_rooms) && (!strcasecmp(name, all_rooms[j])) ) {
		    if (ntvri > MAX_CLIENTS) {
			fprintf(stderr, "<vr> Exceeded MAX_CLIENTS recompile with larger constant\n");
			break;
		    }
		    tvri[ntvri].window = childList[i];
		    tvri[ntvri].parent = parentWindow;
		    tvri[ntvri].room = ALLROOMS;
		    memcpy((String) &tvri[ntvri].wm_state, (String) &wmstate, sizeof(WM_STATE));
		    strcpy( tvri[ntvri].name, name);
		    ntvri++;
		    its_in_all_rooms = True;
		}
	    }
	    if (! its_in_all_rooms) {
		if (ntvri > MAX_CLIENTS) {
		    fprintf(stderr, "<vr> Exceeded MAX_CLIENTS recompile with larger constant\n");
		    break;
		}
		tvri[ntvri].window = childList[i];
		tvri[ntvri].room = room;
		memcpy((String) &tvri[ntvri].wm_state, (String) &wmstate, sizeof(WM_STATE));
		strcpy( tvri[ntvri].name, name);
		ntvri++;
	    }
	}
    }

    XFree(childList);
}

static Bool WindowNotInList(win, list, nlist, found)
Window win;
VRInfo *list;
int nlist;
VRInfo **found; /* set to point at the list element with function returns False */
{
    int i;

    for (i = 0; i < nlist; i++) {
	if (win == list[i].window) {
		*found = &list[i];
		return False;
	}
    }
    return True; /* we could not find the window in the list so it must be new */
}

static void GetGeometrys()
{
    int x, y, i;
    XWindowAttributes atts;
    Window dummywin;

    for (i = 0; i < nvri; i++) {
	if (vri[i].wm_state.state == NormalState) {
	    XGetWindowAttributes(dpy, vri[i].window, &atts);

	    x = atts.x;
	    y = atts.y;

	    XTranslateCoordinates (dpy, vri[i].window, atts.root, 0, 0, &x, &y, &dummywin);

	    vri[i].x = x;
	    vri[i].y = y;
	    vri[i].width = atts.width;
	    vri[i].height = atts.height;
	    printf(" found geometry for %s (window %x) : x = %d y = %d w = %d h = %d\n",
		vri[i].name, vri[i].window, vri[i].x, vri[i].y, vri[i].width, vri[i].height);
	}
    }
}

static void InitRoom(init, room)
Bool init;
int room;
{
    int i, j;
    VRInfo *found = NULL;

    InitRoomAux(DefaultRootWindow(dpy), True, room); 
    /* The tvri (ntvri) array has all the windows that exist over all rooms */

    if (init == False) {

	for (i = 0; i < ntvri; i++) {
	    if (WindowNotInList(tvri[i].window, vri, nvri, &found)) {
		Bool its_in_all_rooms = False;

		/* New window so stamp the room */
		for (j = 0; (j < n_all_rooms) && (its_in_all_rooms == False); j++) {
		    if (!strcasecmp(tvri[i].name, all_rooms[j])) {
			tvri[i].room = ALLROOMS;
			its_in_all_rooms = True;
		    }
		}
		if (! its_in_all_rooms) tvri[i].room = room;
	    }

	    else {
		Bool its_in_all_rooms = False;

		/* Existing window. Make sure room is correct */
		tvri[i].room = found->room;

		/* Check if name has changed such that the window no belongs to ALLROOMS */
		for (j = 0; (j < n_all_rooms) && (its_in_all_rooms == False); j++) {
		    if (!strcasecmp(tvri[i].name, all_rooms[j])) {
			tvri[i].room = ALLROOMS;
			its_in_all_rooms = True;
		    }
		}

		if ( (! its_in_all_rooms) && tvri[i].room == ALLROOMS) {
			/* Window name has changed from something in the all_rooms array to something
			 * not in the array. Stamp the room as current_room since we never kept track
			 * of the room it was in before it's name changed to something in the all_rooms array.
			 * (This is a primitive way of migrating windows to other rooms)
			 */
			tvri[i].room = current_room;
		}

		if (tvri[i].room != room) {
			/* 
			 * If the state has changed for this window in its room 
			 * then register this state change. Otherwise keep the 
			 * state of the window as it was in its room.
			 */
			tvri[i].wm_state.state = found->wm_state.state;
		}
	    }
	}
    }
    nvri = ntvri;
    memcpy((String) vri, (String) tvri, sizeof(vri));
    if (!mwm_running) GetGeometrys();
}

static void up(rd, ud)
XtPointer rd;
XtPointer ud;
{
    int i, room, cur_room;
    int *val = (int *) rd;
    Widget tb = 0;


	switch (*val) {
	  case -1:
		/* 
		 * This is a message to reload vri array and check each member in it for their VROOM property. 
		 * If the vri->room is different than the property then switch the window to the 
		 * room given in the property. If there is no propert then set it.
		 */
		InitRoom(False, current_room);
		for (i = 0; i < nvri; i++) {
		    if (GetVroomProperty(dpy, vri[i].window, &room, &cur_room)) {
			if (room && (vri[i].room != (int) room)) {
				vri[i].room = (int) room;
				SetWindowVisibility(&vri[i]);
			}
		    }
		    else SetVroomProperty(dpy, vri[i].window, vri[i].room, current_room);
		}
	    break;
	  default:
	    tb = roompb[*val-1];
	    break;
	}
	if (tb) {
		if (app_data.verbose) printf(" Extern event to switch to room %d\n", *val);
		XmToggleButtonSetState(tb, True, True);
	}
}

static void SaveRoomTitles()
{
    FILE *fp = NULL;
    char buf[80], filename[256];
    int i;

    strcpy(filename, ExpandFileName(app_data.vrTitleFile));

    fp = fopen(filename, "r");
    if (fp) {
	fclose(fp);
	unlink (filename);
    }

    fp = fopen(filename, "w");
    for (loop = 0; loop < MAX_ROOMS; loop++)
	fprintf(fp, "%s\n", room_label[loop]);
    fclose(fp);
}

static void LoadLabel(to, from)
String to, from;
{
    int i;

    strncpy(to, from, LABEL_LENGTH);
    to[STRING_LENGTH-1] = '\0';
    for (i = 0; i < STRING_LENGTH-1; i++) if (to[i] == '\n') to[i] = '\0';
}

static void InitRoomTitles()
{
    FILE *fp = NULL;
    int loop;
    char buf[80];

      if ((fp = fopen(ExpandFileName(app_data.vrTitleFile), "r")) == NULL) return;
  
     for (loop = 0; loop < MAX_ROOMS; loop++) {
 	if (fgets(buf, sizeof(buf), fp)) {
 	    LoadLabel(room_label[loop], buf);
 	    SetString(roompb[loop], room_label[loop]);
	}
     }
     fclose(fp);
}

static Bool SetRoomLabel(room)
int room;
{
     Widget text, pb;
     String c, label;
     char tlabel[STRING_LENGTH];
     int i;
  
     text = roomtext[room];
     pb = roompb[room];
     label = room_label[room];
  
     c = XmTextFieldGetString(text);
     strncpy(tlabel, c, LABEL_LENGTH);
     tlabel[STRING_LENGTH-1] = '\0';
     for (i = 0; i < STRING_LENGTH-1; i++) if (tlabel[i] == '\n') tlabel[i] = '\0';
     XtFree(c);
 
     if (strcmp(label, tlabel)) {
	strcpy(label, tlabel);
	SetString(pb, label);
	return (True);
    }
    return (False);
}

static Bool SetRoomLabels()
{
    int edited = 0;

    for (loop = 0; loop < MAX_ROOMS; loop++)
	edited += (int) SetRoomLabel(loop);

    return ( (Bool) edited);
}

static void menucb(w, room_to_cfg, cbs)
Widget w;
int room_to_cfg;
XmPushButtonCallbackStruct *cbs;
{
    static Bool first = True;

    if (first) {
	first = False;
	DialogInterface();
    }

    if (XtIsManaged(dialog_form)) {
	XtUnmanageChild(dialog_form);
	if (SetRoomLabels()) SaveRoomTitles();
    }
    else XtManageChild(dialog_form);
}

/*
 * read the vrLoadFile if it exists and load the programs specied there in the specified room
 */
static void LoadPrefs(cd, id)
XtPointer cd;
XtIntervalId *id;
{
    FILE *fp = NULL;
    int i, delay;
    static int room = 0;
    char buf[132], cmd[132];

    if ( (fp = fopen(ExpandFileName(app_data.vrLoadFile), "r")) == NULL ) return;

    while (fgets(buf, sizeof(buf), fp)) {
	    if (buf[0] == '#') continue;
	    delay = 0; room = 0; cmd[0] = '\0';
	    sscanf(buf, " %d %d ", &room, &delay);
	    strcpy(cmd, &buf[strspn(buf, " 0123456789\n")] );
	    for (i = 0; i < strlen(cmd); i++) if (cmd[i] == '\n') cmd[i] = '\0';
	    if (room && ((int) room != current_room)) {
		/* Set the toggle button and change to the desire room */
		up((XtPointer) &room, (XtPointer) 0); 
	    }
	    if (cmd[0]) {
		system(cmd); /* Run the command now that we're in the desired room */
		if (delay) sleep (delay);
	    }
    }
    fclose(fp);
}

static void GetAllRoomPrefs()
{
    FILE *fp = NULL;
    char buf[80], filename[256];
    int i, count = 0;

    strcpy(filename, ExpandFileName(app_data.vrAllRoomsFile));

    fp = fopen(filename, "r");
    if (fp) {
	/* First see how many programs the user has in the vrAllRoomsFile */

	while (fgets(buf, sizeof(buf), fp)) {
	    if (buf[0] == '#') continue;
	    count++;
	}
	rewind(fp);

    }

    /* Now allocate the data structure for array of program names 
     * We allocate space for all the user preferences plus this program
     * and iconbox (for motif)
     */
    all_rooms = (String*) XtCalloc(2 + count, sizeof(String));
    if (all_rooms == NULL) {
	fprintf(stderr, "<vr> Cannot allocate memory! Goodbye\n");
	exit_receiver(1);
    }

    for (i = 0; i < 2 + count ; i++) {
	all_rooms[i] = (String) XtCalloc(80, 1);
	if (all_rooms[i] == NULL) {
	    fprintf(stderr, "<vr> Cannot allocate memory! Goodbye\n");
	    exit_receiver(1);
	}
    }

    /* Now load the array of program names */
    count = 0;
    strcpy(all_rooms[0], title); count ++;
    strcpy(all_rooms[1], "iconbox"); count ++; 

    if (fp) {
	while (fgets(buf, sizeof(buf), fp)) {
	    if (buf[0] == '#') continue;
	    strncpy(all_rooms[count], buf, 80);
	    all_rooms[count][79] = '\0';
		for (i = 0; i < 80; i++)
			if (all_rooms[count][i] == '\n') all_rooms[count][i] = '\0';
	    count++;
	}
	fclose (fp);
    }

    n_all_rooms = count;
}


static void DialogInterface()
{
    Widget btnrc, rc1;
    Widget sep, closepb, helppb;
    Arg args[10];
    char name[30];
    int loop;

    dialog_form = XmCreateFormDialog(pb, "menu", args, 0);

    rc1 = XtVaCreateManagedWidget("rc1", xmRowColumnWidgetClass, dialog_form, 
	XmNorientation, XmHORIZONTAL, 
	XmNpacking, XmPACK_COLUMN,
	XmNnumColumns, MAX_ROOMS,
	XmNisHomogeneous, False, 
	XmNtopAttachment, XmATTACH_FORM, XmNtopOffset, 2, XmNleftAttachment, XmATTACH_FORM, 
	XmNrightAttachment, XmATTACH_FORM, XmNleftOffset, 2, 0);

      XtVaCreateManagedWidget("Title For Room 1  ", xmLabelGadgetClass, rc1, NULL);
  
     roomtext[0] = XtVaCreateManagedWidget("text", xmTextFieldWidgetClass, rc1,
  	XmNcolumns, LABEL_LENGTH, XmNmaxLength, LABEL_LENGTH, NULL);
  
     XmTextFieldSetString(roomtext[0], room_label[0]);
  
  
     for (loop = 1; loop < MAX_ROOMS; loop++)
     {
 	sprintf(name, "Title For Room %d  ", loop+1);
 	XtVaCreateManagedWidget(name, xmLabelGadgetClass, rc1, NULL);
 	roomtext[loop] = XtVaCreateManagedWidget("text", xmTextFieldWidgetClass, rc1,
 						  XmNcolumns, LABEL_LENGTH, XmNmaxLength, LABEL_LENGTH, NULL);
 	XmTextFieldSetString(roomtext[loop], room_label[loop]);
     }
  
    sep = XtVaCreateManagedWidget("sep", xmSeparatorGadgetClass, dialog_form, 
	XmNtopAttachment, XmATTACH_WIDGET, XmNtopWidget, rc1, XmNtopOffset, 5, 
	XmNleftAttachment, XmATTACH_FORM, XmNrightAttachment, XmATTACH_FORM, 
	XmNrightOffset, 2, XmNleftOffset, 2, 0);

    btnrc = XtVaCreateManagedWidget("btnrc", xmRowColumnWidgetClass, dialog_form, 
	XmNtopAttachment, XmATTACH_WIDGET, XmNtopWidget, sep, XmNorientation, XmHORIZONTAL, 
	XmNentryAlignment, XmALIGNMENT_CENTER, XmNleftAttachment, XmATTACH_FORM, 
	XmNbottomAttachment, XmATTACH_FORM, XmNbottomOffset, 5, XmNleftOffset, 2, XmNtopOffset, 5, 0);

    closepb = XtVaCreateManagedWidget("Close", xmPushButtonGadgetClass, btnrc, 
	XmNrecomputeSize, False, XmNwidth, 100, 0);

    XtAddCallback(closepb, XmNactivateCallback, menucb, 0);

    helppb = XtVaCreateManagedWidget("Help", xmPushButtonGadgetClass, btnrc, 
	XmNrecomputeSize, False, XmNwidth, 100, 0);

/*
    XtAddCallback(helppb, XmNactivateCallback, helpsys_invoker, "vroom");
*/
}

static Bool GetWMState(window, wmState)
Window          window;     /* window from which to get WM_STATE */
WM_STATE       *wmState;    /* contents of WM_STATE */
{
    xPropWMState   *prop = NULL;
    Atom            actual_type;
    int             actual_format;
    unsigned long   nitems;
    unsigned long   leftover;
    static Bool first = True;
    static Atom wmStateAtom;

    if (first) {
	first = False;
	wmStateAtom = XInternAtom(dpy, "WM_STATE", False);
    }

    if (XGetWindowProperty(dpy, window, wmStateAtom,
	0L, (long) NumPropWMStateElements, False, wmStateAtom, 
	&actual_type, &actual_format, &nitems, &leftover, (unsigned char **) &prop) != Success)
        return False;

    /* If the property is undefined on the window, return null pointer. */

    if ((actual_type != wmStateAtom) ||
        (nitems < NumPropWMStateElements) || (actual_format != 32)) {
        if (prop != NULL) XFree((String) prop);
        return False;
    }
    wmState->state = (int) prop->state;
    wmState->icon = (Window) prop->icon;
    XFree((String) prop);
    return True;
}

static Bool GetWMName(window, name)
    Window          window;     /* window from which to get WM_STATE */
    String name;
{
    XTextProperty   textProperty;

    XGetWMName(dpy, window, &textProperty);

    if (textProperty.encoding == XA_STRING) {
	strcpy(name, textProperty.value);
	return True;
    }
    else {
	strcpy(name, "");
	if (textProperty.value) XFree((String) textProperty.value);
	return False;
    } 
}

static void SetString(w, str)
Widget w;
String str;
{
	XmString xmstr = XmStringCreateSimple(str);
	XtVaSetValues(w, XmNlabelString, xmstr, NULL);
	XmStringFree(xmstr);
}

static void Usage()
{
    printf("Usage: vr options	where options are:\n");
    printf(" -vrLoadFile file	   (File containing commands to load when vr starts up [$HOME/.vrload])\n");
    printf(" -vrTitleFile file	   (File containing room labels [$HOME/.vrtitle])\n");
    printf(" -vrAllRoomsFile file  (File containing titles that live in all rooms [$HOME/.vrallrooms])\n");
    printf(" -room1_cmds script    (Script containing commands to execute whenever you go into this rom)\n");
    printf(" -room2_cmds script    (Script containing commands to execute whenever you go into this rom)\n");
    printf(" -room3_cmds script    (Script containing commands to execute whenever you go into this rom)\n");
    printf(" -room4_cmds script    (Script containing commands to execute whenever you go into this rom)\n");
    printf(" -room5_cmds script    (Script containing commands to execute whenever you go into this rom)\n");
    printf(" -room6_cmds script    (Script containing commands to execute whenever you go into this rom)\n");
    printf(" -r1c colorname        (Background color of room1 toggle button/root window)\n");
    printf(" -r2c colorname        (Background color of room2 toggle button/root window)\n");
    printf(" -r3c colorname        (Background color of room3 toggle button/root window)\n");
    printf(" -r4c colorname        (Background color of room4 toggle button/root window)\n");
    printf(" -r5c colorname        (Background color of room5 toggle button/root window)\n");
    printf(" -r6c colorname        (Background color of room6 toggle button/root window)\n");
    printf(" -verbose\n");
    printf(" -room1_passwd passwd_file  (Passwd file to use on room1)\n");
    printf(" -room2_passwd passwd_file  (Passwd file to use on room2)\n");
    printf(" -room3_passwd passwd_file  (Passwd file to use on room3)\n");
    printf(" -room4_passwd passwd_file  (Passwd file to use on room4)\n");
    printf(" -room5_passwd passwd_file  (Passwd file to use on room5)\n");
    printf(" -room6_passwd passwd_file  (Passwd file to use on room6)\n");
    printf(" -nrows n 			(Number rows to layout buttons)\n");
    printf(" -nrooms n 			(Number rooms 1-6)\n");
    printf(" -dialog  			(Don't want dialog to relabel buttons)\n");
    printf(" +dialog  			(Do want dialog to relabel buttons)\n");
    exit (0);
}
