/*
 * vrutil.c
 Copyright (c) 1992 by Richard Mauri Trimble Navigation Ltd.

 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
 supporting documentation.

 This file is provided AS IS with no warranties of any kind.  The author
 shall have no liability with respect to the infringement of copyrights,
 trade secrets or any patents by this file or any part thereof.  In no
 event will the author be liable for any lost revenue or profits or
 other special, indirect and consequential damages.
 */
#include <stdio.h>
#include <string.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>

#define PROPERTY_NAME "VROOM"
#define PROPERTY_TYPE "VROOM_TYPE"

typedef struct {
    int current_room;
    int room;
} VROOM;

void SetVroomProperty(dpy, window, room, current_room)
Display *dpy;
Window window;
int room, current_room;
{
    VROOM vroom;
    Atom property, type;
    long length = 0;

    property = XInternAtom(dpy, PROPERTY_NAME, False);
    type = XInternAtom(dpy, PROPERTY_TYPE, False);
    length = sizeof(VROOM);

    vroom.current_room = current_room;
    vroom.room = room;

    XChangeProperty(dpy, window, property, type, 8,
        PropModeReplace, (char *) &vroom, (int) length);

}

Bool GetVroomProperty(dpy, window, room, current_room)
Display *dpy;
Window window;
int *room, *current_room; /* returned */
{
    VROOM *vroom = NULL;
    Atom property, type, actual_type;
    unsigned long nitems = 0, bytes_after = 0;
    long offset = 0, length = 0;
    int status, actual_format;

    property = XInternAtom(dpy, PROPERTY_NAME, False);
    type = XInternAtom(dpy, PROPERTY_TYPE, False);
    length = sizeof(VROOM);

    status = XGetWindowProperty(dpy, window, property, offset, length, False, type,
        &actual_type, &actual_format, &nitems, &bytes_after, &vroom);

    if (vroom) {
	*room = vroom->room;
	*current_room = vroom->current_room;

	XFree(vroom);
	return True;
    }
    return False;
}

